#include "qnmap.h"
#include <qpushbutton.h>

#include <string.h>
#include <qstring.h>

#include <qlabel.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qstringlist.h>
#include <qcheckbox.h>
#include <qmultilineedit.h>

#include <pthread.h>
#include <signal.h>

/* 
 *  Constructs a qnmap which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
qnmap::qnmap( QWidget* parent,  const char* name, WFlags fl )
    : FormQPENmapBase( parent, name, fl )
{
    connect(BtnScan, SIGNAL(clicked()), this, SLOT(scanClicked()));
    connect(BtnClear, SIGNAL(clicked()), this, SLOT(clear()));
    connect(CBScanType, SIGNAL(activated(int)), this, SLOT(idlescan(int)));
    connect(ChPort, SIGNAL(clicked()), this, SLOT(portClicked()));
    connect(ChDecoy, SIGNAL(clicked()), this, SLOT(decoyClicked()));

	top_element=NULL;
	scanning=false;

	CBHost->insertItem("127.0.0.1",-1);
	
	CBScanType->insertItem("SYN",-1);
	CBScanType->insertItem("UDP",-1);
	CBScanType->insertItem("Ping",-1);
	CBScanType->insertItem("IP Protocol",-1);
	CBScanType->insertItem("Connect()",-1);
	CBScanType->insertItem("FIN",-1);
	CBScanType->insertItem("Xmas",-1);
	CBScanType->insertItem("Null",-1);
	CBScanType->insertItem("ACK",-1);
	CBScanType->insertItem("Window",-1);
	CBScanType->insertItem("Idlescan",-1);
	//CBScanType->setCurrentItem(4);
	
	CBPingType->insertItem("None",-1);
	CBPingType->insertItem("TCP ACK",-1);
	CBPingType->insertItem("TCP SYN",-1);
	CBPingType->insertItem("Ping",-1);
	CBPingType->insertItem("Timestamp Request",-1);
	CBPingType->insertItem("Netmask Request",-1);
	CBPingType->insertItem("TCP ACK & Ping",-1);
	
	CBTimeing->insertItem("Paranoid",-1);
	CBTimeing->insertItem("Sneaky",-1);
	CBTimeing->insertItem("Polite",-1);
	CBTimeing->insertItem("Normal",-1);
	CBTimeing->insertItem("Aggressive",-1);
	CBTimeing->insertItem("Insane",-1);
	CBTimeing->setCurrentItem(1);
}

/*  
 *  Destroys the object and frees any allocated resources
 */
qnmap::~qnmap()
{
    // no need to delete child widgets, Qt does it all for us
	//if (scanning) kill(tpid, SIGKILL);
}

void qnmap::appendOutput(const QString text) {
	int line,col;
	mleOutput->getCursorPosition( &line, &col );
	mleOutput->insertAt( text, line, col );
	mleOutput->setCursorPosition(mleOutput->numLines(), 32767);
}

// Clear TreeView
void qnmap::clear() {
	if (scanning) return;
	ListViewScan->clear();	
	mleOutput->clear();
	top_element=NULL;	
}

// PortRange CheckBox is clicked 
void qnmap::portClicked() {
	if (ChPort->isChecked()) {
		LEPort->setEnabled(true);
		LEPort->selectAll();
		LEPort->setFocus();
	} else {
		LEPort->clearFocus();
		LEPort->setEnabled(false);
	}
}


// DecoyHost CheckBox is clicked 
void qnmap::decoyClicked() {
	if (ChDecoy->isChecked()) {
		LEDecoy->setEnabled(true);
		LEDecoy->selectAll();
		LEDecoy->setFocus();
	} else {
		LEDecoy->clearFocus();
		LEDecoy->setEnabled(false);
	}

}

// Idlescan Type is selected
void qnmap::idlescan(int n) {
		if (n==10) {
			LEZombie->setEnabled(true);
			LEZombie->setText("zombie_host[:port]");
			LEZombie->selectAll();
			LEZombie->setFocus();
			return;
		} else if (n==2 && CBPingType->currentItem()==0) {
			CBPingType->setCurrentItem(1);
		}
		LEZombie->clearFocus();
		LEZombie->setEnabled(false);
}

// Scan Button clicked
void qnmap::scanClicked() {
	if (scanning) return;

	pthread_create(&tpid, NULL, runit, (void *)this);
}

// funct. for pthread_create
// don't know how to call qnmap::scan() directly
void* runit(void* arg) {
	qnmap* caller=(qnmap*)arg;
	caller->scan();
	return(0);
}

// Scan
void qnmap::scan() {
	
	if (scanning) return;
	scanning=true;

	char cmd[512];
	char result[256];

	FILE *pipe;
	QListViewItem *element;
	QListViewItem *parent;

	LScan->setText("Scanning...");
	/* add target to CBHost list */
	addhost(CBHost->currentText().ascii());
	
	getcmd(cmd);
	//printf("cmd: %s\n", cmd);
	
	// print cmd to TextViewOut
	appendOutput(QString(cmd) + '\n');
	
	/* run nmap & read output */
	if ((pipe = popen(cmd, "r")) == NULL) {
		snprintf(result, 256, "Error: Can't run %s", cmd);
		appendOutput(result);
		return;
	}

	/* parse output and display in ListViewScan */
	while(fgets(result, 256, pipe) != NULL) {

		/* put result into TextViewOutput */
		appendOutput(result);

		/* new port ? */
		if( strstr(result, "Adding open port ") != NULL ) {
			char* tmp=&result[16];
			char proto[4];
			bool newitem=true;

			if ( strstr(tmp, "/tcp") != NULL ) {
				strstr(tmp, "/tcp")[0]=0;
				strcpy(proto, "TCP");
			} else if ( strstr(tmp, "/udp") != NULL ) {
				strstr(tmp, "/udp")[0]=0;
				strcpy(proto, "UDP");
			}
			element=parent->firstChild();
			while ( element != NULL && newitem==true) {
				if ( strcmp(element->text(0).ascii(), tmp)==0 &&
						strcmp(element->text(1).ascii(), proto)==0 ) newitem=false;
				element=element->nextSibling();
			}
			if (newitem) new QListViewItem(parent, tmp, proto);
			
		/* new host ? */
		} else if( strstr(result, " appears to be up") != NULL ) { 
			char *ip_addr, *host;
			host=result+5;
			ip_addr=strstr(host, " ");
			*ip_addr=0; ip_addr+=2;
			*(strstr(ip_addr, " ")-1)=0;

			if ( top_element!=NULL ) {
				bool found=false;
				element=top_element;
				while (element != NULL && !found) { 
					if (strcmp(element->text(0).ascii(), ip_addr)==0) {
						parent=element;
						found=true;
					}
					element=element->nextSibling();
				}
				if (!found) {
					element=new QListViewItem(ListViewScan, ip_addr, host);
					element->setOpen(true);
					parent=element;
				}
			} else {
				element=new QListViewItem(ListViewScan, ip_addr, host);
				element->setOpen(true);
				top_element=element;
				parent=element;
		    }


		/* IP-Protocol scan */
		} else if( strstr(result, "Initiating IPProto Scan against ") != NULL ) { 
			while(fgets(result, 256, pipe) != NULL && strcmp(result,"\n") !=0 ) {
					
				/* put result into TextViewOutput */
				appendOutput(result);

				if ( strstr(result, "  open  ") != NULL ) {
					char *proto;
					char tmp[16];
					bzero(tmp, 16);
					bool newitem=true;

					proto=strstr(result, " open ")+13;
					strstr(proto, " ")[0]=0;
					strstr(result, " ")[0]=0;

					
					strncpy(tmp, proto, 16);
					strncat(tmp, " (", 16);
					strncat(tmp, result, 16);
					strncat(tmp, ")", 16);

					element=parent->firstChild();
					while ( element != NULL && newitem==true) {
						if ( strcmp(element->text(1).ascii(), tmp)==0 )
								newitem=false;
						element=element->nextSibling();
					}
					if (newitem) new QListViewItem(parent, "", tmp);
				}
			}

		}

	}

	appendOutput("\n============================================\n");
	LScan->setText("");
	scanning=false;
}

// add current target host to CBHost if not exists
bool qnmap::addhost(const char *host) {
	int i;

	for (i=0; i<CBHost->count(); ++i) {
		if (strcmp(host, CBHost->text(i).ascii())==0) 
			return false;
	}

	CBHost->insertItem(host, -1);
	return true;
}
// assamble nmap cmd
void qnmap::getcmd(char *cmd) {
	const char *nmapcmd="nmap -v -v";
	char scan[4];
	char ping[4];
	char options[64];
	char port[64];
	char decoy[128];
	char timeing[16];
	char zombie[32];
	
	bzero(options, 64);
	bzero(port, 64);
	bzero(decoy, 128);
	bzero(timeing, 16);
	bzero(zombie, 32);

	/* get Scan Type */
	switch(CBScanType->currentItem()) {
		case 0:			// SYN
			strncpy(scan, "-sS", 4); 
			break;
		case 1:			// UDP
			strncpy(scan, "-sU", 4); 
			break;
		case 2:			// Ping
			strncpy(scan, "-sP", 4); 
			break;
		case 3:			// IP Protocol
			strncpy(scan, "-sO", 4); 
			break;
		case 4:			// Connect()
			strncpy(scan, "-sT", 4); 
			break;
		case 5:			// FIN
			strncpy(scan, "-sF", 4); 
			break;
		case 6:			// Xmas
			strncpy(scan, "-sX", 4); 
			break;
		case 7:			// Null
			strncpy(scan, "-sN", 4); 
			break;
		case 8:			// ACK
			strncpy(scan, "-sA", 4); 
			break;
		case 9:			// Window
			strncpy(scan, "-sW", 4); 
			break;
		case 10:		// Idlescan
			strncpy(scan, "-sI", 4);
			zombie[0]=' ';
			strncat(zombie, LEZombie->text().ascii(), 32);
			break;
		}

	/* get Ping Type */
	switch(CBPingType->currentItem()) {
		case 0:			// None
			strncpy(ping, "-P0", 4); 
			break;
		case 1:			// TCP ACK
			strncpy(ping, "-PT", 4); 
			break;
		case 2:			// TCP SYN
			strncpy(ping, "-PS", 4); 
			break;
		case 3:			// Ping
			strncpy(ping, "-PI", 4); 
			break;
		case 4:			// Timestamp Req
			strncpy(ping, "-PP", 4); 
			break;
		case 5:			// Netmask Req
			strncpy(ping, "-PM", 4); 
			break;
		case 6:			// TCP ACK & Ping
			strncpy(ping, "-PB", 4); 
			break;
	}

	/* get options */
	if (ChOS->isChecked()) strncat(options, " -O", 64);
	if (ChIdent->isChecked()) strncat(options, " -I", 64);
	if (ChRPC->isChecked()) strncat(options, " -sR", 64);
	if (ChFrag->isChecked()) strncat(options, " -f", 64);
	if (ChFast->isChecked()) strncat(options, " -F", 64);
	if (ChNoDNS->isChecked()) strncat(options, " -n", 64);
	if (ChRandHosts->isChecked()) strncat(options, " --randomize_hosts", 64);
	if (ChRandom->isChecked()) {
			strncat(options, " -iR", 64);
			/*const QString empty("");
			CBHost->setCurrentText(empty);*/
	}

	/* get port range & decoy */
	if (ChPort->isChecked()) {
		strcpy(port, " -p ");
		strncat(port, LEPort->text().ascii(), 64);
	}
	if (ChDecoy->isChecked() && LEDecoy->text().length()>0) {
		strcpy(decoy, " -D");
		strncat(decoy, LEDecoy->text().ascii(), 128);
	}

	/* get timeing */
	if (ChTimeing->isChecked()) {
		strcpy(timeing, " -T ");
		switch(CBTimeing->currentItem()) {
			case 0:			// Paranoid
				strncat(timeing, "Paranoid", 16);
				break;
			case 1:			// Sneaky
				strncat(timeing, "Sneaky", 16);
				break;
			case 2:			// Polite
				strncat(timeing, "Polite", 16);
				break;
			case 3:			// Normal
				strncat(timeing, "Normal", 16);
				break;
			case 4:			// Aggressive
				strncat(timeing, "Aggressive", 16);
				break;
			case 5:			// Insane
				strncat(timeing, "Insane", 16);
				break;
		}
	}

	/* assemble nmap cmd */
	snprintf(cmd, 512, "%s %s%s %s%s%s%s%s %s 2>&1", nmapcmd, scan, zombie, ping, 
			options, timeing, port, decoy, CBHost->currentText().latin1() ); 
}
